/*	      RS232-RC5-Decoder mit Tiny13, basierend auf                   */
/*             SUART und RC5 von Peter Dannegger					    */
/*                   nderung: nur TX-UART                              */


/************************************************************************/
/*                                                                      */
/*                      Software UART                                   */
/*                                                                      */
/*              Author: Peter Dannegger                                 */
/*                      danni@specs.de                                  */
/*                                                                      */
/************************************************************************/
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include "rc5.h"

#define F_CPU 9600000UL
#include <util/delay.h>

#define XTAL	9600e3	//mglichst Vielfaches von 1.8432MHz !

#define	BAUD	9600

#define STX     	PB1     // OC0B
#define STXDDR  	DDRB

#define LED_PORT	PORTB
#define LED_DDR		DDRB
#define LED			2

#define IR_PIN		PINB
#define IR_DDR		DDRB
#define IR			0

#define BIT_TIME	(unsigned short)((XTAL + BAUD/2) / BAUD)

volatile unsigned char stx_counter;
unsigned char stx_data;

void board_init(void);
void suart_init(void);
void sputchar(unsigned char);
void sputs(unsigned char*);

int main( void )
{
	board_init();
    suart_init();
    sei();
    sputs((unsigned char*)"Tiny13 kann senden!\n\r");

	unsigned char i = 0;
	
	LED_PORT &= ~(1<<LED);		// LED einschalten
	for (i=0; i<100; i++)		// 200ms warten
	{	
		_delay_ms(2);
	}
	LED_PORT |= (1<<LED);		// LED ausschalten
	for (i=0; i<100; i++)		// 200ms warten
	{	
		_delay_ms(2);
	}
	LED_PORT &= ~(1<<LED);		// LED einschalten
	for (i=0; i<100; i++)		// 200ms warten
	{	
		_delay_ms(2);
	}
	LED_PORT |= (1<<LED);		// LED ausschalten
	for (i=0; i<100; i++)		// 200ms warten
	{	
		_delay_ms(2);
	}
		
	for(;;)
	{
		unsigned int i;
		char s[30];
		cli();
		i = rc5_data;			// read two bytes from interrupt !
    	rc5_data = 0;
    	sei();
    	if( i )
		{
      		//DDRB = i;				// LED output
      		//sputchar(( i >> 11 & 1) + '0');	// Toggle Bit
      		//sputchar(' ');
			itoa( i >> 6 & 0x1F, s, 10);	// Device address
			//sputs( s );
			//sputchar(' ');
			itoa((i & 0x3F) | (~i >> 7 & 0x40), s, 10); // Key Code
			//sputs( s );
			//sputs( "\n\r" );

			unsigned char j = 0;
			LED_PORT &= ~(1<<LED);		// LED einschalten
			for (j=0; j<100; j++)		// 100ms warten
			{	
				_delay_ms(1);
			}
			LED_PORT |= (1<<LED);		// LED ausschalten
		}
	}
}

void board_init(void)
{
	LED_DDR |= (1<<LED);
	IR_DDR &= ~(1<<IR);

	//TCCR0B |= (1<<CS02);			//divide by 256
	TCCR0B |= (1<<CS00);			//divide by 1
	TIMSK0 |= (1<<TOIE0);			//enable timer interrupt
	TIFR0 &= ~((1<<OCF0B)|(1<<OCF0A)|(1<<TOV0));	//Reset IRQ-Flags
	cyclecount = 0;

	//sei();		//IRQs aktivieren
}

void suart_init( void )
{
  OCR0B = TCNT0 + 1;			// force first compare
  TCCR0A = 1<<COM0B1^1<<COM0B0;		// set OC1A high, T1 mode 0
  TIMSK0 |= (1<<OCIE0B);

  stx_counter = 0;			// nothing to sent
  STXDDR |= 1<<STX;			// TX output
}

void sputchar(unsigned char val )			// send byte
{
  while( stx_counter );			// until last byte finished
  stx_data = ~val;			// invert data for Stop bit generation
  stx_counter = 10;			// 10 bits: Start + data + Stop
}

void sputs(unsigned char* txt)			// send string
{
  while( *txt )
    sputchar( *txt++ );
}

ISR( SIG_OUTPUT_COMPARE0B )		// tx bit
{
  unsigned  dout;
  unsigned char mycount;

  OCR0B += BIT_TIME;			// next bit slice
  mycount = stx_counter;

  if( mycount ){
    mycount = --stx_counter;		// count down
    dout = 1<<COM0B1;			// set low on next compare
    if( mycount != 9 ){			// no start bit
      if( !(stx_data & 1) )		// test inverted data
	dout = 1<<COM0B1^1<<COM0B0;	// set high on next compare
      stx_data >>= 1;			// shift zero in from left
    }
    TCCR0A |= dout;
  }
}
